(ns damp.ekeko
   (:refer-clojure :exclude [==])
   (:use clojure.core.logic)
   (:use [damp.ekeko jdtnode logic gui mli reification basic]))




(set! *warn-on-reflection* true)

(defmacro ekeko [logicvars & goals]
  `(run* [resultvar#] 
         (fresh [~@logicvars]
                (equals resultvar# [~@logicvars])
                ~@goals)))

(defmacro ekeko* [vars & goals]
  `(let [querystr# (str '(ekeko* [~@vars] ~@goals))
         start# (System/nanoTime)
         resultsqc# (run* [resultvar#] 
                          (fresh [~@vars]
                                 (equals resultvar# [~@vars])
                                 ~@goals))
         elapsed#  (/ (double (- (System/nanoTime) start#)) 1000000.0)
         cnt# (count resultsqc#)]
     (eclipse-uithread-return (fn [] (open-barista-results-viewer* querystr# '(~@vars) resultsqc# elapsed# cnt#)))))
             



;Example repl session
(comment
  (use 'damp.ekeko)
  (in-ns 'damp.ekeko)
 (ekeko* [?t ?b] (ast :TypeDeclaration ?t) (equals ?b (.resolveBinding ?t)))
  (ekeko* [?m] (ast :MethodDeclaration ?m))
 
  (ekeko* [?cu] (ast :CompilationUnit ?cu))
  (ekeko* [?cu] (ast :StringLiteral ?cu))
  (ekeko* [?cu ?key ?child] (ast :CompilationUnit ?cu) (has ?key ?cu ?child))
  
  (ekeko* [?cu ?key ?child ?shouldbecu ?shouldbekey] (ast :CompilationUnit ?cu) (has ?key ?cu ?child) (has ?shouldbekey ?shouldbecu ?child))
  (ekeko* [?node ?nodetype ?childtype ?child] (ast ?nodetype ?node) (has ?childtype ?node ?child))
  
  (ekeko* [?node ?child ?keyword] (child ?keyword ?node ?child)) 
  
  (ekeko* [?cu ?exp] (ast :MethodDeclaration ?cu) (child+ ?cu ?exp) (succeeds (actual-expression? ?exp)))   
  
  
  (ekeko* [?inv] (ast :MethodInvocation ?inv) (has :expression ?inv nil))
  
  (ekeko* [?inv ?child] (ast :MethodInvocation ?inv) (child :arguments ?inv ?child))

  (ekeko* [?m ?o] (ast :MethodDeclaration ?m) (fresh [?os]
                                                  (equals ?os (method-overriders ?m))
                                                  (contains ?os ?o)))
  
  (ekeko* [?i ?m] (ast :MethodInvocation ?i) (fresh [?ts]
                                                  (equals ?ts (invocation-targets ?i))
                                                  (contains ?ts ?m)))
  
  (ekeko* [?import] (ast :ImportDeclaration ?import))
  
  
    
 
  )

